import board
import storage
import os
import time
import digitalio
import busio
import displayio
import terminalio
from adafruit_display_text import label
import adafruit_ssd1327

err = "-"

try:
    import sdioio
    sd_type = 0
except:
    pass
    #print( "sdioio library not available" )

try:
    import sdcardio
    sd_type = 1
except:
    pass
    #print( "sdcardio library not available" )


if sd_type == 0:
    sdcard = sdioio.SDCard(
        clock=board.SDIO_CLOCK,
        command=board.SDIO_COMMAND,
        data=board.SDIO_DATA,
        frequency=25000000)
    print( "sdioio success" )
    vfs = storage.VfsFat(sdcard)

if sd_type == 1:
    cs = board.SD_CS
    sd_card_spi = busio.SPI(board.SD_SCK, MOSI=board.SD_MOSI, MISO=board.SD_MISO)
    try:
        sdcard = sdcardio.SDCard(sd_card_spi, cs)
        vfs = storage.VfsFat(sdcard)
        print( "sdcardio success" )
    except:
        err = "not found"
        print( err )

        vfs = False

if vfs:
    storage.mount(vfs, "/sd")
    sdcard_status = os.statvfs("/sd")
    sdfbsize = sdcard_status[0]
    sdfbavail = sdcard_status[4]
    storage_free_percent = sdfbavail/sdfbsize *100
    print( sdcard_status )
    print( "bytes per block = {}".format(sdfbsize))
    print( "free blocks = {}".format(sdfbavail))
    sdbytesavail_MB = sdfbavail * sdfbsize/ 1000000
    print( "MB available = {}".format(sdbytesavail_MB))
    sdfssize = sdcard_status[2]
    sdbytessize_MB = int (round(( sdfssize * sdfbsize/ 1000000 ), 0))
    print( "MB drive size = {}".format(sdbytessize_MB))
    sdbytessize_GB = int( round( sdbytessize_MB /1000, 0 ))
    sdavail_percent = int( sdbytesavail_MB/ sdbytessize_MB * 100)
    if sdbytessize_GB < 1:
        print( "drive space available = {} % of {} MB".format(sdavail_percent, sdbytessize_MB))
    else:
        print( "drive space available = {} % of {} GB".format(sdavail_percent, sdbytessize_GB))
    line_bytes_size = 1000
    line_capacity_remaining = int(sdbytesavail_MB * 1000000/ line_bytes_size)
    sampling_interval_s = 1.0
    lines_per_s= 1/sampling_interval_s
    time_remaining_h = int( line_capacity_remaining * lines_per_s /3600 )
    time_remaining_days = round(time_remaining_h/24, 1)
    print( "data collection time remaining until this drive is full = {} h = {} days".format(time_remaining_h, time_remaining_days))
displayio.release_displays()

try:
    i2c_bus = board.I2C()
    print( "i2c bus initialized" )
except:
    print( "i2c bus failed to initialize" )
    i2c_bus = False

try:
    display_bus = displayio.I2CDisplay(i2c_bus, device_address=0x3D)
    display = adafruit_ssd1327.SSD1327(display_bus, width=128, height=128)
    print( "display initialized" )
except:
    print( "display failed to initialize" )
    display = False

loop_count = 0

if display:
    display_group = displayio.Group()
    display.show( display_group )
    background_bitmap = displayio.Bitmap( 128, 128, 1 )
    background_palette = displayio.Palette(1)
    background_palette[0] = 0x888888  # Gray
    background_square = displayio.TileGrid( background_bitmap, pixel_shader=background_palette, x=0, y=0 )
    display_group.append(background_square)

    text = "SD card"
    text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
    text_group = displayio.Group( scale=2, x=20, y=20 )
    text_group.append( text_area )
    display_group.append( text_group )

    text = ""
    status_text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
    status_text_group = displayio.Group( scale=2, x=10, y=50 )
    status_text_group.append( status_text_area )
    display_group.append( status_text_group )


    text = "{}".format( loop_count )
    text_area = label.Label( terminalio.FONT, text=text, color=0xFFFFFF )
    text_group = displayio.Group( scale=2, x=40, y=110 )
    text_group.append( text_area )
    display_group.append( text_group )

    if vfs:
        if sdbytessize_GB < 1:
            status_text_area.text = "{}% free\nof {} MB".format(sdavail_percent, sdbytessize_MB)
        else:
            status_text_area.text = "{}% free\nof {} GB".format(sdavail_percent, sdbytessize_GB)

try:
    while loop_count < 2:
        if display:
            text_area.text = str(loop_count)
        print("loop count {}".format(loop_count))
        #time.sleep(0.1)
        if vfs:
            with open( "/sd/test.txt", "w" ) as f:
                f.write( "year, month, day, batch\n" )
                f.write( "2022, 12, 30, 0\n" )


            with open("/sd/test.txt", "r") as f:
                print("Printing lines in file:")
                #print()
                line = f.readline()
                while line != '':
                    print(line)
                    line = f.readline()
                text_area.text = "PASS"
        else:
            text_area.text = "FAIL"

        #print()
        loop_count += 1
        time.sleep(1)
finally:
    if display:
        time.sleep(0.5)
    if vfs:
        sdcard.deinit()
        print( "sd card deinitialized" )
    displayio.release_displays()
    print( "displayio displays released" )
    i2c_bus.deinit()
    print( "i2c_bus deinitialized" )
    print( " ---- end --- " )
